==> paddleslim-2.6.0-py3-none-any.whl: paddleslim/lc/quantizers/quant_func.py <==
import itertools
import paddle
from paddleslim_ops import quant_blockwise, dequant_blockwise

def create_dynamic_map(signed=True, max_exponent_bits=7, total_bits=8):
    """
    Creates the dynamic quantiztion map.

    The dynamic data type is made up of a dynamic exponent and
    fraction. As the exponent increase from 0 to -7 the number
    of bits available for the fraction shrinks.

    This is a generalization of the dynamic type where a certain
    number of the bits and be reserved for the linear quantization
    region (the fraction). n determines the maximum number of
    exponent bits.

    For more details see
    (8-Bit Approximations for Parallelism in Deep Learning)[https://arxiv.org/abs/1511.04561]
    """

    data = []
    # these are additional items that come from the case
    # where all the exponent bits are zero and no
    # indicator bit is present
    non_sign_bits = total_bits - (1 if signed else 1)
    additional_items = 2 ** (non_sign_bits - max_exponent_bits) - 1
    for i in range(max_exponent_bits):
        fraction_items = int((2 ** (i + non_sign_bits - max_exponent_bits) + 1 if signed else 2 ** (i + non_sign_bits - max_exponent_bits + 1) + 1))
        boundaries = paddle.linspace(0.1, 1, fraction_items)
        means = (boundaries[:-1] + boundaries[1:]) / 2.0
        data += ((10 ** (-(max_exponent_bits - 1) + i)) * means).tolist()
        if signed:
            data += (-(10 ** (-(max_exponent_bits - 1) + i)) * means).tolist()

    if additional_items > 0:
        boundaries = paddle.linspace(0.1, 1, additional_items + 1)
        means = (boundaries[:-1] + boundaries[1:]) / 2.0
        data += ((10 ** (-(max_exponent_bits - 1) + i)) * means).tolist()
        if signed:
            data += (-(10 ** (-(max_exponent_bits - 1) + i)) * means).tolist()

    data.append(0)
    data.append(1.0)

    assert len(data) == 2**total_bits

    gap = 256 - len(data)
    for i in range(gap):
        data.append(0)

    data.sort()
    return data

def create_fp8_map(signed=True, exponent_bits=5, precision_bits=2, total_bits=8):
    e = exponent_bits
    p = precision_bits
    has_sign = 1 if signed else 0
    assert e+p == total_bits-has_sign
    # the exponent is biased to 2^(e-1) -1 == 0
    evalues = []
    pvalues = []
    for i, val in enumerate(range(-((2**(exponent_bits-has_sign))), 2**(exponent_bits-has_sign), 1)):
        evalues.append(2**val)


    values = []
    lst = list(itertools.product([0, 1], repeat=precision_bits))
    #for ev in evalues:
    bias = 2**(exponent_bits-1)
    for evalue in range(2**(exponent_bits)):
        for bit_pattern in lst:
            value = (1 if evalue != 0 else 0)
            for i, pval in enumerate(list(bit_pattern)):
                value += pval*(2**-(i+1))
            if evalue == 0:
                # subnormals
                value = value*2**-(bias)
            else:
                # normals
                value = value*2**-(evalue-bias-1)
            values.append(value)
            if signed:
                values.append(-value)


    assert len(values) == 2**total_bits
    values.sort()
    if total_bits < 8:
        gap = 256 - len(values)
        for i in range(gap):
            values.append(0)
    values.sort()
    code = paddle.to_tensor(values)
    code /= code.max()

    return code


def quantize_nf4(x, blocksize):
    return quant_blockwise(x, None, blocksize=blocksize, quant_type="nf4")

def quantize_fp4(x, blocksize):
    return quant_blockwise(x, None, blocksize=blocksize, quant_type="fp4")

def dequantize_nf4(x, absmax, blocksize):
    return dequant_blockwise(x, None, absmax, blocksize=blocksize,  quant_type="nf4")

def dequantize_fp4(x, absmax, blocksize):
    return dequant_blockwise(x, None, absmax, blocksize=blocksize,  quant_type="fp4")

def quantize_8bit(x, code, blocksize, quant_type="fp8"):
    if code is None:
        if quant_type=="fp8":
            code = create_fp8_map(signed=True, exponent_bits=2, precision_bits=1, total_bits=4)
        else:
            code = paddle.to_tensor(create_dynamic_map())
    return quant_blockwise(x, code, blocksize=blocksize, quant_type="8bit")

def dequantize_8bit(x, code, absmax, blocksize, quant_type="fp8"):
    if code is None:
        if quant_type=="fp8":
            code = create_fp8_map(signed=True, exponent_bits=2, precision_bits=1, total_bits=4)
        else:
            code = paddle.to_tensor(create_dynamic_map())

    return dequant_blockwise(x, code, absmax, blocksize=blocksize, quant_type="8bit")

==> paddleslim-2.6.4.post823-py3-none-any.whl: paddleslim/lc/quantizers/quant_func.py <==
import itertools
import paddle
from paddleslim_ops import quant_blockwise, dequant_blockwise

def create_dynamic_map(signed=True, max_exponent_bits=7, total_bits=8):
    """
    Creates the dynamic quantiztion map.

    The dynamic data type is made up of a dynamic exponent and
    fraction. As the exponent increase from 0 to -7 the number
    of bits available for the fraction shrinks.

    This is a generalization of the dynamic type where a certain
    number of the bits and be reserved for the linear quantization
    region (the fraction). n determines the maximum number of
    exponent bits.

    For more details see
    (8-Bit Approximations for Parallelism in Deep Learning)[https://arxiv.org/abs/1511.04561]
    """

    data = []
    # these are additional items that come from the case
    # where all the exponent bits are zero and no
    # indicator bit is present
    non_sign_bits = total_bits - (1 if signed else 1)
    additional_items = 2 ** (non_sign_bits - max_exponent_bits) - 1
    for i in range(max_exponent_bits):
        fraction_items = int((2 ** (i + non_sign_bits - max_exponent_bits) + 1 if signed else 2 **
                             (i + non_sign_bits - max_exponent_bits + 1) + 1))
        boundaries = paddle.linspace(0.1, 1, fraction_items)
        means = (boundaries[:-1] + boundaries[1:]) / 2.0
        data += ((10 ** (-(max_exponent_bits - 1) + i)) * means).tolist()
        if signed:
            data += (-(10 ** (-(max_exponent_bits - 1) + i)) * means).tolist()

    if additional_items > 0:
        boundaries = paddle.linspace(0.1, 1, additional_items + 1)
        means = (boundaries[:-1] + boundaries[1:]) / 2.0
        data += ((10 ** (-(max_exponent_bits - 1) + i)) * means).tolist()
        if signed:
            data += (-(10 ** (-(max_exponent_bits - 1) + i)) * means).tolist()

    data.append(0)
    data.append(1.0)

    assert len(data) == 2 ** total_bits

    gap = 256 - len(data)
    for i in range(gap):
        data.append(0)

    data.sort()
    return data

def create_fp8_map(signed=True, exponent_bits=5, precision_bits=2, total_bits=8):
    """
    创建FP8映射，用于将FP32数值映射到8位的二进制表示形式。
    
    Args:
        signed (bool): 是否是有符号型。默认值为True。
        exponent_bits (int): 表示指数位数。默认值为5。
        precision_bits (int): 表示精度位数。默认值为2。
        total_bits (int): 需要生成的总位数。默认值为8。
    
    Returns:
        Tensor: 生成的FP8映射，类型为paddle.Tensor。
    
    """
    e = exponent_bits
    p = precision_bits
    has_sign = 1 if signed else 0
    assert e + p == total_bits-has_sign
    # the exponent is biased to 2^(e-1) -1 == 0
    evalues = []
    pvalues = []
    for i, val in enumerate(range(-((2 ** (exponent_bits-has_sign))), 2**(exponent_bits-has_sign), 1)):
        evalues.append(2 ** val)


    values = []
    lst = list(itertools.product([0, 1], repeat=precision_bits))
    #for ev in evalues:
    bias = 2 ** (exponent_bits-1)
    for evalue in range(2 ** (exponent_bits)):
        for bit_pattern in lst:
            value = (1 if evalue != 0 else 0)
            for i, pval in enumerate(list(bit_pattern)):
                value += pval * (2**-(i+1))
            if evalue == 0:
                # subnormals
                value = value * 2**-(bias)
            else:
                # normals
                value = value * 2**-(evalue-bias-1)
            values.append(value)
            if signed:
                values.append(-value)


    assert len(values) == 2 ** total_bits
    values.sort()
    if total_bits < 8:
        gap = 256 - len(values)
        for i in range(gap):
            values.append(0)
    values.sort()
    code = paddle.to_tensor(values)
    code /= code.max()

    return code


def quantize_nf4(x, blocksize):
    """
    量化函数，用于对输入张量进行量化操作。
    
    Args:
        x (torch.Tensor): 需要量化的输入张量，类型为 torch.Tensor。
        blocksize (int): 表示量化块大小的参数。
    
    Returns:
        torch.Tensor: 经过量化操作后的张量，与输入张量具有相同的形状和数据类型。
    
    """
    return quant_blockwise(x, None, blocksize=blocksize, quant_type="nf4")

def quantize_fp4(x, blocksize):
    """
    将FP32数据量化为FP4，使用量化块大小为blocksize。
    
    Args:
        x (torch.Tensor): 输入的张量，维度为[B, C // blocksize, H, W]或[B, N].
        blocksize (int): 量化块的大小，范围是1-8。
    
    Returns:
        torch.Tensor: 返回一个经过量化处理后的张量，类型为torch.float32。
    
    """
    return quant_blockwise(x, None, blocksize=blocksize, quant_type="fp4")

def dequantize_nf4(x, absmax, blocksize):
    """
    逐块量化并解码一个Numpy数组。
    
    Args:
        x (np.ndarray): 需要被解码的Numpy数组。
        absmax (float): 绝对最大值，用于将输入数组映射为0~255范围内的整数值。
        blocksize (int): 每个分块的大小。
    
    Returns:
        np.ndarray: 经过量化、逐块解码后的Numpy数组。
    
    """
    return dequant_blockwise(x, None, absmax, blocksize=blocksize, quant_type="nf4")

def dequantize_fp4(x, absmax, blocksize):
    """
    逐块量化浮点数，使用FP4的量化方法。
    
    Args:
        x (torch.Tensor): 需要量化的输入张量，形状为[B, C, H, W]或[N, C, H, W]。
        absmax (int): 量化范围的上限，应为正整数。
        blocksize (int): 每个通道的量化大小，应为2的幂。
    
    Returns:
        torch.Tensor: 经过量化处理后的输出张量，形状与输入相同。
    
    """
    return dequant_blockwise(x, None, absmax, blocksize=blocksize, quant_type="fp4")

def quantize_8bit(x, code, blocksize, quant_type="fp8"):
    """
    量化输入张量到指定比特位数的FP32或INT32张量。
    
    Args:
        x (Tensor): 输入张量，可以是任何维度的张量，只需保证最后一维度为通道维度。
        code (Tensor|None): 指定量化的映射表，若未指定则根据quant_type自动生成映射表。默认为None。
        blocksize (int): 块大小，表示每个块的量化步数。
        quant_type (str): 量化类型，支持"fp8"和"8bit"两种模式，默认为"fp8"。
    
    Returns:
        Tensor: 量化后的FP32或INT32张量。
    
    """
    if code is None:
        if quant_type=="fp8":
            code = create_fp8_map(signed=True, exponent_bits=2, precision_bits=1, total_bits=4)
        else:
            code = paddle.to_tensor(create_dynamic_map())
    return quant_blockwise(x, code, blocksize=blocksize, quant_type="8bit")

def dequantize_8bit(x, code, absmax, blocksize, quant_type="fp8"):
    """
    量化解码函数，输入数据为paddle.Tensor类型，返回解码后的数据。
    
    Args:
        x (paddle.Tensor): 待解码的数据。
        code ([int]): 量化参数映射表。
        absmax ([float]): 梯度最大绝对值。
        blocksize ([int]): 数据块大小。
        quant_type (str): 量化模式，默认值为"fp8"。
    
    Returns:
        paddle.Tensor: 解码后的数据。
    """
    if code is None:
        if quant_type=="fp8":
            code = create_fp8_map(signed=True, exponent_bits=2, precision_bits=1, total_bits=4)
        else:
            code = paddle.to_tensor(create_dynamic_map())

    return dequant_blockwise(x, code, absmax, blocksize=blocksize, quant_type="8bit")
